#!/usr/bin/bash

# Include the prefixdevname tools only if this was explicitely requested
check() {
    return 255
}

install() {
    inst /usr/lib/udev/prefixdevname
    inst_rules 71-prefixdevname.rules
}
